@@ -2,7 +2,7 @@ from dataclasses import asdict, dataclass, field, replace, astuple
 from enum import Enum
 from functools import wraps
 from inspect import get_annotations, signature, Parameter
-from typing import Optional, Callable, Final, cast, Any, TypeVar
+from typing import Optional, Callable, Final, cast, Any, TypeVar, overload
 from types import FunctionType
 from algosdk.abi import Method
 from pyteal import (
@@ -32,6 +32,7 @@ from pyteal import (
 from beaker.state import AccountStateValue, ApplicationStateValue
 
 HandlerFunc = Callable[..., Expr]
+DecoratorFunc = Callable[[HandlerFunc], HandlerFunc]
 
 _handler_config_attr: Final[str] = "__handler_config__"
 
@@ -394,9 +395,23 @@ def _remove_self(fn: HandlerFunc) -> HandlerFunc:
     return fn
 
 
+@overload
+def internal(
+    return_type_or_handler: HandlerFunc,
+) -> HandlerFunc:
+    ...
+
+
+@overload
+def internal(
+    return_type_or_handler: TealType,
+) -> DecoratorFunc:
+    ...
+
+
 def internal(
     return_type_or_handler: TealType | HandlerFunc,
-) -> HandlerFunc | Callable[..., HandlerFunc]:
+) -> HandlerFunc | DecoratorFunc:
     """creates a subroutine to be called by logic internally
 
     Args:
@@ -436,21 +451,45 @@ def internal(
     return _impl
 
 
+@overload
 def external(
-    fn: HandlerFunc | None = None,
+    func: HandlerFunc,
     /,
     *,
     name: str | None = None,
     authorize: SubroutineFnWrapper | None = None,
     method_config: MethodConfig | None = None,
     read_only: bool = False,
-) -> HandlerFunc | Callable[..., HandlerFunc]:
+) -> HandlerFunc:
+    ...
+
+
+@overload
+def external(
+    *,
+    name: str | None = None,
+    authorize: SubroutineFnWrapper | None = None,
+    method_config: MethodConfig | None = None,
+    read_only: bool = False,
+) -> DecoratorFunc:
+    ...
+
+
+def external(
+    func: HandlerFunc | None = None,
+    /,
+    *,
+    name: str | None = None,
+    authorize: SubroutineFnWrapper | None = None,
+    method_config: MethodConfig | None = None,
+    read_only: bool = False,
+) -> HandlerFunc | DecoratorFunc:
 
     """
     Add the method decorated to be handled as an ABI method for the Application
 
     Args:
-        fn: The function being wrapped.
+        func: The function being wrapped.
         name: Name of ABI method. If not set, name of the python method will be used.
             Useful for method overriding.
         authorize: a subroutine with input of ``Txn.sender()`` and output uint64
@@ -482,10 +521,10 @@ def external(
 
         return fn
 
-    if fn is None:
+    if func is None:
         return _impl
 
-    return _impl(fn)
+    return _impl(func)
 
 
 def bare_external(
@@ -554,13 +593,33 @@ def is_bare(fn: HandlerFunc) -> bool:
     )
 
 
+@overload
+def create(
+    fn: HandlerFunc,
+    /,
+    *,
+    authorize: SubroutineFnWrapper | None = None,
+    method_config: Optional[MethodConfig] | None = None,
+) -> HandlerFunc:
+    ...
+
+
+@overload
+def create(
+    *,
+    authorize: SubroutineFnWrapper | None = None,
+    method_config: Optional[MethodConfig] | None = None,
+) -> DecoratorFunc:
+    ...
+
+
 def create(
     fn: HandlerFunc | None = None,
     /,
     *,
     authorize: SubroutineFnWrapper | None = None,
     method_config: Optional[MethodConfig] | None = None,
-) -> HandlerFunc | Callable[..., HandlerFunc]:
+) -> HandlerFunc | DecoratorFunc:
     """set method to be handled by an application call with its :code:`OnComplete`
         set to :code:`NoOp` call and ApplicationId == 0
 
@@ -590,7 +649,7 @@ def create(
         else:
             return external(method_config=MethodConfig(**mconfig), authorize=authorize)(
                 fn
-            )  # type: ignore
+            )
 
     if fn is None:
         return _impl
@@ -598,9 +657,21 @@ def create(
     return _impl(fn)
 
 
+@overload
+def delete(
+    fn: HandlerFunc, /, *, authorize: SubroutineFnWrapper | None = None
+) -> HandlerFunc:
+    ...
+
+
+@overload
+def delete(*, authorize: SubroutineFnWrapper | None = None) -> DecoratorFunc:
+    ...
+
+
 def delete(
     fn: HandlerFunc | None = None, /, *, authorize: SubroutineFnWrapper | None = None
-) -> HandlerFunc | Callable[..., HandlerFunc]:
+) -> HandlerFunc | DecoratorFunc:
     """set method to be handled by an application call with it's
         :code:`OnComplete` set to :code:`DeleteApplication` call
 
@@ -622,9 +693,7 @@ def delete(
             return external(
                 method_config=MethodConfig(delete_application=CallConfig.CALL),
                 authorize=authorize,
-            )(
-                fn
-            )  # type: ignore
+            )(fn)
 
     if fn is None:
         return _impl
@@ -632,9 +701,21 @@ def delete(
     return _impl(fn)
 
 
+@overload
+def update(
+    fn: HandlerFunc, /, *, authorize: SubroutineFnWrapper | None = None
+) -> HandlerFunc:
+    ...
+
+
+@overload
+def update(*, authorize: SubroutineFnWrapper | None = None) -> DecoratorFunc:
+    ...
+
+
 def update(
     fn: HandlerFunc | None = None, /, *, authorize: SubroutineFnWrapper | None = None
-) -> HandlerFunc | Callable[..., HandlerFunc]:
+) -> HandlerFunc | DecoratorFunc:
     """set method to be handled by an application call with it's
         :code:`OnComplete` set to :code:`UpdateApplication` call
 
@@ -656,9 +737,7 @@ def update(
             return external(
                 method_config=MethodConfig(update_application=CallConfig.CALL),
                 authorize=authorize,
-            )(
-                fn
-            )  # type: ignore
+            )(fn)
 
     if fn is None:
         return _impl
@@ -666,9 +745,21 @@ def update(
     return _impl(fn)
 
 
+@overload
+def opt_in(
+    fn: HandlerFunc, /, *, authorize: SubroutineFnWrapper | None = None
+) -> HandlerFunc:
+    ...
+
+
+@overload
+def opt_in(*, authorize: SubroutineFnWrapper | None = None) -> DecoratorFunc:
+    ...
+
+
 def opt_in(
     fn: HandlerFunc | None = None, /, *, authorize: SubroutineFnWrapper | None = None
-) -> HandlerFunc | Callable[..., HandlerFunc]:
+) -> HandlerFunc | DecoratorFunc:
     """set method to be handled by an application call with it's
            :code:`OnComplete` set to :code:`OptIn` call
 
@@ -689,9 +780,7 @@ def opt_in(
         else:
             return external(
                 method_config=MethodConfig(opt_in=CallConfig.CALL), authorize=authorize
-            )(
-                fn
-            )  # type: ignore
+            )(fn)
 
     if fn is None:
         return _impl
@@ -699,9 +788,21 @@ def opt_in(
     return _impl(fn)
 
 
+@overload
+def clear_state(
+    fn: HandlerFunc, /, *, authorize: SubroutineFnWrapper | None = None
+) -> HandlerFunc:
+    ...
+
+
+@overload
+def clear_state(*, authorize: SubroutineFnWrapper | None = None) -> DecoratorFunc:
+    ...
+
+
 def clear_state(
     fn: HandlerFunc | None = None, /, *, authorize: SubroutineFnWrapper | None = None
-) -> HandlerFunc | Callable[..., HandlerFunc]:
+) -> HandlerFunc | DecoratorFunc:
     """set method to be handled by an application call with it'ws
         :code:`OnComplete` set to :code:`ClearState` call
 
@@ -723,9 +824,7 @@ def clear_state(
             return external(
                 method_config=MethodConfig(clear_state=CallConfig.CALL),
                 authorize=authorize,
-            )(
-                fn
-            )  # type: ignore
+            )(fn)
 
     if fn is None:
         return _impl
@@ -733,9 +832,21 @@ def clear_state(
     return _impl(fn)
 
 
+@overload
+def close_out(
+    fn: HandlerFunc, /, *, authorize: SubroutineFnWrapper | None = None
+) -> HandlerFunc:
+    ...
+
+
+@overload
+def close_out(*, authorize: SubroutineFnWrapper | None = None) -> DecoratorFunc:
+    ...
+
+
 def close_out(
     fn: HandlerFunc | None = None, /, *, authorize: SubroutineFnWrapper | None = None
-) -> HandlerFunc | Callable[..., HandlerFunc]:
+) -> HandlerFunc | DecoratorFunc:
     """set method to be handled by an application call with it's
         :code:`OnComplete` set to :code:`CloseOut` call
 
@@ -757,9 +868,7 @@ def close_out(
             return external(
                 method_config=MethodConfig(close_out=CallConfig.CALL),
                 authorize=authorize,
-            )(
-                fn
-            )  # type: ignore
+            )(fn)
 
     if fn is None:
         return _impl
@@ -767,9 +876,21 @@ def close_out(
     return _impl(fn)
 
 
+@overload
+def no_op(
+    fn: HandlerFunc, /, *, authorize: SubroutineFnWrapper | None = None
+) -> HandlerFunc:
+    ...
+
+
+@overload
+def no_op(*, authorize: SubroutineFnWrapper | None = None) -> DecoratorFunc:
+    ...
+
+
 def no_op(
     fn: HandlerFunc | None = None, /, *, authorize: SubroutineFnWrapper | None = None
-) -> HandlerFunc | Callable[..., HandlerFunc]:
+) -> HandlerFunc | DecoratorFunc:
     """set method to be handled by an application call with
         it's :code:`OnComplete` set to :code:`NoOp` call
 
@@ -790,9 +911,7 @@ def no_op(
         else:
             return external(
                 method_config=MethodConfig(no_op=CallConfig.CALL), authorize=authorize
-            )(
-                fn
-            )  # type: ignore
+            )(fn)
 
     if fn is None:
         return _impl
